/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BPNG_CONFIGURATION_H
#define BPNG_CONFIGURATION_H 1

#include <bp/ng/list.h>
#include <bp/ng/corpus.h>


/**
 * Runtime configuration specified in a configuration file (*.bp.conf).
 */
struct configuration {
   struct list morph_names;
   struct list header_names;
   struct list meta_keys;
   struct list xmlns;
   char *locale;
};

/**
 * Read the configuration file for the corpus.
 */
void configuration_create(struct configuration *, const char *base_name);

/**
 * Destroy the configuration data.
 */
void configuration_destroy(struct configuration *);

/**
 * Commit the configuration data to the binary corpus.
 */
void configuration_commit(const struct configuration *, struct corpus *);

#endif
