/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#include "text-stack.h"


void text_stack_init(struct text_stack *this)
{
   this->buffer = NULL;
}

void text_stack_create(struct text_stack *this)
{
   this->buffer = malloc(16);
   this->buffer_size = 32;
   this->stack_size = 8;

   this->buffer = malloc(this->buffer_size);
   this->stack = malloc(this->stack_size * sizeof *this->stack);

   this->buffer_current = 0;
   this->stack_current = 0;
}

void text_stack_destroy(struct text_stack *this)
{
   if (this->buffer) {
      free(this->buffer);
      free(this->stack);
      this->buffer = NULL;
   }
}

void text_stack_reserve(struct text_stack *this, size_t len)
{
   if (this->buffer_current + len >= this->buffer_size) {
      do {
         this->buffer_size *= 2;
      } while (this->buffer_current + len >= this->buffer_size);
      this->buffer =
         realloc(this->buffer, this->buffer_size * sizeof *this->buffer);
   }
}

void text_stack_push(struct text_stack *this, const char *buf, size_t len)
{
   if (this->stack_current == this->stack_size) {
      this->stack_size *= 2;
      this->stack =
         realloc(this->stack, this->stack_size * sizeof *this->stack);
   }
   this->stack[this->stack_current++] = this->buffer_current;
   text_stack_reserve(this, len + 1);
   memcpy(this->buffer + this->buffer_current, buf, len);
   this->buffer_current += len;
   this->buffer[this->buffer_current] = '\0';
}

void
text_stack_push2(struct text_stack *this, const char *buf, size_t len,
   int sep)
{
   if (this->stack_current == this->stack_size) {
      this->stack_size *= 2;
      this->stack =
         realloc(this->stack, this->stack_size * sizeof *this->stack);
   }
   text_stack_reserve(this, len + 2);
   this->stack[this->stack_current++] = this->buffer_current;
   this->buffer[this->buffer_current++] = (char) sep;
   memcpy(this->buffer + this->buffer_current, buf, len);
   this->buffer_current += len;
   this->buffer[this->buffer_current] = '\0';
}

void text_stack_pop(struct text_stack *this)
{
   if (this->stack_current > 0) {
      this->buffer_current = this->stack[--this->stack_current];
      this->buffer[this->buffer_current] = '\0';
   }
   else
      abort();
}

const char *text_stack_text(struct text_stack *this)
{
   return this->buffer;
}


