/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BP_SYNTAX_PARSER_H
#define BP_SYNTAX_PARSER_H

#include <poliqarp-config.h>

#include <libxml/parser.h>

#include <sakura/common/hash-table.h>

#include "list.h"

struct group {
   int id;      /**< Index used to distinguish this group from other groups
                     with the same type. */
   char *type;  /**< The type of this group. */
};

struct syntax_parser {
   xmlSAXHandler sax_handler;
   xmlParserCtxtPtr xml_parser;
   int num_errors;
   const char *dirname;
   const char *filename;
   struct marena arena;
   struct hash_table htfrom, htto, htsynh, htsemh;
   bool have_syntax;
};

/** Creates a parser for syntax.xml files. */
struct bp_parser *syntax_parser_create(const char *base_name);

#endif /* BP_SYNTAX_PARSER_H */
