/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BPNG_FS_WALK_H
#define BPNG_FS_WALK_H 1

typedef int (*fs_walk_callback)(const char *path, void *extra);

/**
 * Directory tree walk with callback function.
 *
 * For each directory in the directory tree rooted at top (including top
 * itself, but excluding "." and ".."), call callback(dir_name, extra). If the
 * callback returns a non-zero values, no further subdirectories are visited.
 *
 * @note It is fine to pass NULL as extra.
 */
void fs_walk(const char *top, fs_walk_callback callback, void *extra);

#endif
