/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

package ipipan.poliqarp.gui;

import ipipan.poliqarp.logic.Corpus;
import ipipan.poliqarp.logic.MetaData;
import ipipan.poliqarp.util.Pair;
import java.util.HashSet;
import java.util.List;

public class MetaDataConstraintSet extends HashSet<MetaDataConstraintModel>
{
   public MetaDataConstraintSet(Corpus corpus)
   {
      super();
      try {
         List<Pair<MetaData.Type, String> > types = corpus.getMetaTypes();
         for (Pair<MetaData.Type, String> entry : types) {
            switch (entry.first()) {
               case TEXT:
                  add(new TextConstraintModel(entry.second()));
                  break;
               case DATE:
                  add(new DateConstraintModel(entry.second()));
                  break;
            }
         }            
      } catch (Exception e) {}
   }
   
   public String getQueryConstraint()
   {
      boolean one = false;
      StringBuilder res = new StringBuilder("(");
      for (MetaDataConstraintModel m : this) {
         String s = m.getQueryConstraint();
         if (s != null) {
            res.append(s);
            res.append(" & ");
            one = true;
         }
      }
      if (one)
         res.delete(res.length() - 3, res.length());
      res.append(")");
      return one ? res.toString() : null;
   }
}
