/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

package ipipan.poliqarp.gui;

import java.awt.event.*;
import javax.swing.*;
import com.jgoodies.forms.layout.CellConstraints;

public class TextConstraint extends MetaDataConstraint
{
   private TextConstraintModel model;
   private boolean active;
   private JCheckBox cbActive;
   private JLabel label;
   private JTextField tfConstraint;
   
   public TextConstraint(TextConstraintModel model)
   {
      this.model = model;
      String text = null;
      active = model.isActive();
      cbActive = new JCheckBox(text, active);
      label = new JLabel(model.getName() + ": ");
      tfConstraint = new JTextField(model.getConstraint(), 20);
      label.setEnabled(active);
      tfConstraint.setEnabled(active);
      cbActive.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e)
         {
            active = !active;
            label.setEnabled(active);
            tfConstraint.setEnabled(active);
         }
      });
   }

   public void update()
   {
      model.setActive(active);
      model.setConstraint(tfConstraint.getText());
   }

   public void setWidgets(JPanel panel, int row)
   {
      CellConstraints cc = new CellConstraints();
      panel.add(cbActive, cc.xy(2, row + 1));
      panel.add(label, cc.xy(4, row + 1));
      panel.add(tfConstraint, cc.xywh(6, row + 1, 7, 1));
   }
}
