/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

package ipipan.poliqarp.logic.results;

import ipipan.poliqarp.connection.PoliqarpConnection;

public class MatchColumn extends ColumnType
{
   protected boolean haveTags = false;
   protected boolean haveLemmata = false;
   
   protected void parseDescription(String desc)
   {
      if (desc.indexOf("l") != -1)
         haveLemmata = true;
      if (desc.indexOf("t") != -1)
         haveTags = true;
   }

   public MatchColumn(String description)
   {
      parseDescription(description);
   }

   public ResultCell readCell(PoliqarpConnection connection) 
      throws java.io.IOException
   {
      return new MatchCell(this, connection);
   }
}
