/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#include <foostring/foostring.h>

#include "foolog.h"
#include "viewport.h"

void foolog(enum msg_type type, const char *message, ...)
{
   va_list ap;
   va_start(ap, message);
   foolog_vformat(type, message, ap);
   va_end(ap);
}

void foolog_vformat(enum msg_type type, const char *message, va_list ap)
{
   char *message_text = string_avformat(message, ap);
   vp_fprintf(type == MSG_ERROR ? vp_err : vp_out, "%s\n", message_text);
   vp_flush(type == MSG_ERROR ? vp_err : vp_out);
   free(message_text);
   if (type == MSG_FATAL)
      exit(EXIT_FAILURE);
}
