/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#include <assert.h>

#include <bp/ng/document.h>
#include <bp/common.h>

void document_create(struct document *this, const char *dir_name,
   const char *morph_name, const char *header_name)
{
   this->dir_name = bp_strdup(dir_name);
   assert(morph_name != NULL);
   assert(header_name != NULL);
   this->morph_name = morph_name;
   this->header_name = header_name;
   this->xml_header = NULL;
   this->meta_keys = NULL;
}

struct document *document_alloc(const char *dir_name, 
   const char *morph_name, const char *header_name)
{
   struct document *this = bp_malloc(sizeof *this); 
   document_create(this, dir_name, morph_name, header_name);
   return this;
}

void document_destroy(struct document *document)
{
   if (document == NULL)
      return;
   free(document->dir_name);
   assert(document->xml_header == NULL);
}

void document_free(void *this)
{
   document_destroy(this);
   free(this);
}
