/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BPNG_INTERP_H
#define BPNG_INTERP_H 1

#include <poliqarp-config.h>

/**
 * A segment interpretation
 */
struct interp {
   char *base; /**< base name */
   char *tag; /**< tag */
   bool disamb; /**< still correct after disambiguation? */
};

/**
 * Initialize the interpretation.
 */
void interp_create(struct interp *);

/**
 * Allocate memory for an interpretation. Then, initialize it.
 */
struct interp *interp_alloc(void);

/**
 * Destroy the interpretation.
 *
 * @note interp#base string will be freed if it is not NULL.
 * @note interp#tag string will be freed if it is not NULL.
 */
void interp_destroy(struct interp *);

/**
 * Destroy the interpretation and free the allocated memory.
 *
 * @note interp#base string will be freed if it is not NULL.
 * @note interp#tag string will be freed if it is not NULL.
 */
void interp_free(void *);

#endif
