/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

package ipipan.poliqarp.gui;

import java.util.EventObject;

/**
 * An event that occurs with respect to a table model. It carries data
 * about: <ul>
 *   <li>the configuration to be used to retrieve texts for the components 
 *       that have to be redrawn as a result of this event;</li>
 *   <li>whether the structure of the table has been changed.</li>
 * </ul>
 */
public class ResultEvent extends EventObject
{
   private Configuration cfg;
   private boolean changed;

   /**
    * Constructs an event bearing the given information.
    * 
    * @param model the source of this event
    * @param cfg the configuration 
    * @param changed whether table structure has changed
    */
   public ResultEvent(ResultModel model, Configuration cfg, boolean changed)
   {
      super(model);
      this.cfg = cfg;
      this.changed = changed;
   }

   /**
    * Returns the source of this event as a <code>ResultModel</code>.
    */
   public ResultModel getSource()
   {
      return (ResultModel)super.getSource();
   }

   /**
    * Returns the configuration that should be used when updating text for a
    * component.
    */
   public Configuration getConfig()
   {
      return cfg;
   }

   /**
    * Answers whether this event was triggered due to the table changing structure
    * (altering number of columns).
    */
   public boolean tableChanged()
   {
      return changed;
   }
}
