/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BPNG_SEGMENT_H
#define BPNG_SEGMENT_H 1

#include <poliqarp-config.h>

#include <sakura/common/hash-table.h>

#include <bp/ng/interp.h>
#include <bp/ng/list.h>

/**
 * A segment.
 */
struct segment {
   bool space; /**< is there a preceding space? */
   char *orth; /**< orthographic form */
   struct list interps; /**< list of possible interpretations */
   struct hash_table *ht;
};

/**
 * Initialize the segment.
 *
 * @param need_id_index true if interpretations could have non-NULL identifiers
 */
void segment_create(struct segment *, bool need_id_index);

/**
 * Destroy the segment.
 *
 * @note segment#orth string will be free()d if it is not NULL.
 */
void segment_destroy(struct segment *);

/**
 * Add the interpretation to the segment.
 *
 * @param id interpretation identifier, or NULL
 */
void segment_add_interp(struct segment *, struct interp *, const char *id);

/**
 * Query for a interpretation of the segment.
 *
 * @param id interpretation identifier
 */
struct interp *segment_get_interp(struct segment *, const char *id);

#endif
