/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BP_TAG_LOOKUP_H
#define BP_TAG_LOOKUP_H

enum xml_tag {
   TOK_CHUNK_LIST,
   TOK_CHUNK,
   TOK_TOK,
   TOK_ORTH,
   TOK_LEX,
   TOK_BASE,
   TOK_CTAG,
   TOK_NS,
   TOK_OTHER,
   TOK_GROUP,
   TOK_UNKNOWN
};

void tag_lookup_init();
enum xml_tag lookup_tag(const char *text);

#endif /* BP_TAG_LOOKUP_H */
