/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

package ipipan.poliqarp.gui;

import java.awt.Color;
import java.awt.Component;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * This cell renderer has mostly standard behavior, but assigns different
 * colors to different rows of the table it renders. More precisely, it is
 * supplied a number of colors upon creation, and each row is assigned the
 * color equal to its number modulo number of colors.
 */
public class MultiColorCellRenderer extends DefaultTableCellRenderer 
{
   private Color[] colors;
   
   /**
    * Constructs a <code>MultiColorCellRenderer</code> with given colors.
    */
   public MultiColorCellRenderer(Color[] colors)
   {
      super();
      this.colors = colors;
   }
   
   /**
    * Returns the multi-color cell renderer.
    */
   public Component getTableCellRendererComponent(JTable table, Object value,
      boolean isSelected, boolean hasFocus, int row, int column)
   {
      setBackground(colors[row % colors.length]);
      return super.getTableCellRendererComponent(table, value, isSelected,
         hasFocus, row, column);
   }
}
