/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#include <assert.h>
#include <stdlib.h>

#include <bp/common.h>
#include <bp/ng/interp.h>

void interp_create(struct interp *this)
{
   assert(this != NULL);
   this->disamb = false;
   this->base = NULL;
   this->tag = NULL;
}

struct interp *interp_alloc(void)
{
   struct interp *result = bp_malloc(sizeof *result);
   interp_create(result);
   return result;
}

void interp_destroy(struct interp *this)
{
   free(this->base);
   this->base = NULL;
   free(this->tag);
   this->tag = NULL;
}

void interp_free(void *this)
{
   interp_destroy(this);
   free(this);
}
