/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

package ipipan.poliqarp.gui;

import java.io.File;
import java.util.regex.*;
import javax.swing.*;
import javax.swing.filechooser.*;

/** 
 * A filter for the corpora files openable by Poliqarp. This implementation 
 * allows only *.poliqarp.corpus.image files to be opened (an arbitrary 
 * choice, may be modified later).
 */
public class CorpusFileFilter extends FileFilter 
{
   private Configuration config;
   private static final String extension = ".poliqarp.corpus.image";
   private static final Pattern fileNamePattern = 
      Pattern.compile(".*" + extension.replace(".", "\\."));

   /**
    * Length of the extension of corpus files.
    */
   public static final int extensionLength = extension.length();

   /** 
    * Creates a new corpus file filter using the specified configuration. 
    */
   public CorpusFileFilter(Configuration config) 
   {
      this.config = config;
   }

   /** 
    * Accepts all directories and *.poliqarp.corpus.image files. 
    */
   public boolean accept(File f) 
   {
      if (f.isDirectory())
         return true;
      return fileNamePattern.matcher(f.getName()).matches();
   }

   /** 
    * Returns the description of this filter. 
    */
   public String getDescription()
   {
      return config.getComponentName("TEXT_CORPORA");
   }
}
