/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

package ipipan.poliqarp.gui;

import ipipan.poliqarp.logic.Match;
import ipipan.poliqarp.logic.Query;
import ipipan.poliqarp.logic.results.ColumnType;

public class CSVResultWriter extends ResultWriter
{
   public CSVResultWriter(Configuration cfg)
   {
      super(cfg);
   }

   private void writeCell(String x, boolean comma) 
   {
      writer.write('"');
      writer.write(x);
      writer.write('"');
      if (comma) 
         writer.write(',');
      else
         writer.println();    
   }

   protected void writeHeader(Query query, String encoding) 
   {
      writeCell(cfg.getComponentName("TEXT_RESULTS"), true);
      writeCell(query.toString(), false);
      int i = 0;
      for (String th : colHeaders) {
         writeCell(th, (i < numCols - 1));
         i++;
      }
   }
    
   protected void writeMatch(Match match)
   {
      for (int i = 0; i < numCols; i++)
         writeCell(match.getCell(i).toCSV(cfg), (i < numCols - 1));
   }
}
