/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BPNG_XML_NKJP
#define BPNG_XML_NKJP 1

#include <poliqarp-config.h>

#include <bp/ng/corpus.h>
#include <bp/ng/document.h>
#include <bp/ng/xml-parser.h>

extern const char *xml_ns_tei;
extern const char *xml_ns_nkjp;

/**
 * Parse the document morphosyntax data in the NKJP variant of the TEI format,
 * for inclusion in the given corpus.
 *
 * @return 0 in case of success, -1 in case of error
 */
int xml_parse_morph_nkjp(xml_reader *, struct document *, struct corpus *);

/**
 * Check if the XML document looks like it is in the TEI format.
 */
bool xml_has_nkjp_root(xml_reader *reader);

#endif
