/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2004-2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef HAVE_TEXT_STACK_H
#define HAVE_TEXT_STACK_H

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

struct text_stack {
   size_t buffer_size;
   size_t stack_size;
   char *buffer;
   size_t *stack;
   size_t stack_current;
   size_t buffer_current;
};

void text_stack_init(struct text_stack *this);

void text_stack_create(struct text_stack *this);

void text_stack_destroy(struct text_stack *this);

void text_stack_reserve(struct text_stack *this, size_t len);

void text_stack_push(struct text_stack *this, const char *buf, size_t len);

void
text_stack_push2(struct text_stack *this, const char *buf, size_t len,
   int sep);

void text_stack_pop(struct text_stack *this);

const char *text_stack_text(struct text_stack *this);

#endif

