/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <foostring/foostring.h>

#include <bp/ng/fs-walk.h>
#include <bp/common.h>

void fs_walk(const char *path, fs_walk_callback callback, void *extra)
{
   DIR *dir = opendir(path);
   if (dir == NULL) {
      bp_warning("unable to descend to directory %s", path);
      return;
   }
   if (callback(path, extra) != 0)
      return;
   struct dirent *entry;
   while ((entry = readdir(dir)) != NULL) {
      if (strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0) 
         continue;
#ifdef _DIRENT_HAVE_D_TYPE
      /* glibc-specific optimization */
      if (entry->d_type != DT_UNKNOWN && entry->d_type != DT_LNK) {
         if (entry->d_type == DT_DIR) {
            char *child_path = string_aformat("%s/%s", path, entry->d_name);
            fs_walk(child_path, callback, extra);
            free(child_path);
         }
      }
      else
#endif
      {
         struct stat statbuf;
         char *child_path = string_aformat("%s/%s", path, entry->d_name);
         if (stat(child_path, &statbuf) != 0)
            bp_warning("unable to get status of file/directory %s", child_path);
         else if (S_ISDIR(statbuf.st_mode))
            fs_walk(child_path, callback, extra);
         free(child_path);
      }
   }
   closedir(dir);
}

