/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>

#include <errno.h>
#include <bp/common.h>
#include <bp/ng/list.h>

void list_append(struct list *this, void *item)
{
   assert(this != NULL);
   if (this->length >= this->capacity) {
      this->capacity = this->capacity * 2 + 1;
      if (this->capacity > SIZE_MAX / sizeof *this->items) {
         errno = ENOMEM;
         bp_system_error("memory allocation error");
      }
      this->items = bp_realloc(this->items, this->capacity * sizeof *this->items);
   }
   this->items[this->length++] = item;
}

void *list_pop(struct list *this)
{
   assert(this != NULL);
   assert(this->length > 0);
   this->length--;
   return this->items[this->length];
}

void list_clear(struct list *this, list_destroy_fn fn)
{
   assert(this != NULL);
   if (fn != NULL) {
      size_t i;
      for (i = 0; i < this->length; i++)
         fn(this->items[i]);
   }
   this->length = 0;
}

void list_destroy(struct list *this, list_destroy_fn fn)
{
   if (this == NULL)
      return;
   list_clear(this, fn);
   free(this->items);
   this->items = NULL;
   this->capacity = 0;
}
