/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations. )
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#include <bp/ng/xml-parser.h>
#include <bp/ng/xpath.h>
#include <libxml/parserInternals.h>

void xml_init()
{
   LIBXML_TEST_VERSION
}

static const int xml_parser_options =
   XML_PARSE_NOCDATA |
   XML_PARSE_NOENT |
   XML_PARSE_COMPACT;

xml_document *xml_document_new(const char *path)
{
   /* TODO: support bzip2-compressed files */
   return xmlReadFile(path, NULL, xml_parser_options);
}

void xml_document_free(xml_document *document)
{
   xmlFreeDoc(document);
}

xml_reader *xml_reader_new(const char *path)
{
   /* TODO: support bzip2-compressed files */
   return xmlReaderForFile(path, NULL, xml_parser_options);
}

void xml_reader_free(xml_reader *reader)
{
   xmlFreeTextReader(reader);
}

void xml_cleanup()
{
   xpath_cleanup();
   xmlCleanupParser();
}

void xml_normalize_space(char *source)
{
   bool space = false;
   char *target = source;

   /* Skip leading whitespaces */
   while (IS_BLANK_CH(*source))
      source++;

   /* Collapse intermediate whitespaces, and skip trailing whitespaces */
   space = false;
   while (*source != '\0') {
      if (IS_BLANK_CH(*source))
         space = true;
      else {
         if (space) {
            *target++ = ' ';
            space = false;
         }
         *target++ = *source;
      }
      source++;
   }
   *target = '\0';
}
